# ioBroker

*...domesticate the Internet of Things.*

See [ioBroker wiki](https://github.com/ioBroker/ioBroker/wiki/Home-(English)) for more information


[ioBroker website](http://iobroker.net)
[Bug tracking JIRA](http://iobroker.net:8000)
[Demo admin](http://iobroker.net:8081)
[Demo VIS](https://iobroker.net:8080)


ioBroker is an integration platform for the [Internet of Things](http://en.wikipedia.org/wiki/Internet_of_Things),
focused on Building Automation, Smart Metering, Ambient Assisted Living, Process Automation, Visualization and
Data Logging. It aims to be a possible replacement for software like f.e. [fhem](http://fhem.de),
[OpenHAB](http://www.openhab.org/) or [the thing system](http://thethingsystem.com/) by the end of 2014. ioBroker will
be the successor of [CCU.IO](http://ccu.io), a project quite popular in the
[german HomeMatic community](http://homematic-forum.de/forum/).

## Concept

ioBroker is not just an application, it's more of a a concept, a database schema, and offers a very easy way for systems
to interoperate. ioBroker defines some common rules for a pair of databases used to exchange data and publish events
between different systems.

![architecture](img/architecture.png)

#### Databases

ioBroker uses [Redis](http://redis.io/) and [CouchDB](http://couchdb.apache.org/). Redis is an in-memory key-value data
store and also a message broker with publish/subscribe pattern. It's used to maintain and publish all states of
connected systems. CouchDB is used to store rarely changing and larger data, like metadata of systems and things,
configurations or any additional files.

#### Adapters

Systems are attached to ioBrokers databases via so called adapters, technically processes running anywhere
in the network and connecting all kinds of systems to ioBrokers databases. A connection to ioBrokers databases can be
implemented in nearly any programming language on nearly any platform and an adapter can run on any host that is able to
reach the databases via ip networking.

A library module for fast and comfortable adapter development exists for Javascript/Node.js until now. Libraries for
adapter development in other languages are planned (python, java, perl, ...).



| adapter    	                                                                                            | description                                                                     	                                                                                                                                                                                                        | status 	    |
|-------------------------------------------------------------------------------------------------------    |----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------   |------------   |
|                                                                                                           | **system services**                                                                                                                                                                                                                                                                       |               |
| [admin](https://github.com/iobroker/ioBroker.nodejs/blob/master/adapter/admin/README.md)      	        | The admin user interface. Comfortably install and update adapters, manage adapter instance configuration and more                                                           	                                                                                                            | 0.6.5  	    |
| [history](https://github.com/iobroker/ioBroker.nodejs/blob/master/adapter//history/README.md)    	        | Manage state history in a two-staged process. A configurable amount of datapoints is kept in Redis for high-speed access, long-time archiving is written to CouchDB with a configurable retention-time. Configuration can be done individually for each state.                            | **beta** 	    |
| [socketio](https://github.com/iobroker/ioBroker.socketio)   	                                            | [Socket.io](http://socket.io) Server                                                            	                                                                                                                                                                                        | 1.0.0         |
| [web](https://github.com/iobroker/ioBroker.nodejs/blob/master/web/README.md)        	                    | [Express](http://expressjs.com/) based webserver                                           	                                                                                                                                                                                            | 1.0.2         |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **automation**                                                                                                                                                                                                                                                                            |               |
| [logic](https://github.com/smiling-Jack/ioBroker.logic/blob/master/README.md) 	                        | Graphical Javascript IDE ([ScriptGUI](https://github.com/smiling-Jack/CCU-IO.ScriptGUI))                                                     	                                                                                                                                            | planned       |
| [javascript](https://github.com/iobroker/ioBroker.nodejs/blob/master/javascript/README.md) 	            | Javascript and CoffeeScript script engine. Use Javascripts/Node.js asynchronous programming with comfortable methods to subscribe to events and schedule events with sophisticated scheduling-features.                                                      	                            | 1.0.3         |
| virtual  	                                                                                                | Map states into virtual namespaces and apply basic rules on the mappings (average, sum, min, max, offset, factor, toggle, timeout, history pattern matching and more).                                         	                                                                        | planned       |
| scenes     	                                                                                            | Create and sequence scenes                                              	                                                                                                                                                                                                                | 0.2.1         |
| trigger     	                                                                                            | Trigger simple actions, scenes or scripts by state change pattern recognition ("reverse-scenes")                                             	                                                                                                                                            | planned       |
| python                                                                                                    | Python script engine                                                      	                                                                                                                                                                                                            | planned 2016  |
| php 	                                                                                                    | PHP script engine                                                      	                                                                                                                                                                                                                | planned 2016  |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **user interfaces**                                                                                                                                                                                                                                                                       |               |
| [mobile](https://github.com/iobroker/ioBroker.mobile)        	                                            | [jQuery Mobile](http://jquerymobile.com/) based User Interface ([yahui](https://github.com/hobbyquaker/yahui) successor)                                             	                                                                                                                    | alpha         |
| [vis](https://github.com/iobroker/ioBroker.vis)        	                                                | Template based HTML5 User Interface (Successor of [DashUI](https://github.com/hobbyquaker/DashUI) - the successful UI-Builder-Framework of [CCU.IO](http://ccu.io). View [DashUI Demos](http://dashui.ccu.io/)                                          	                                | 0.7.4 	    |
| [slim](https://github.com/iobroker/ioBroker.slim)        	                                                | Lightweight javascript library to build custom User Interfaces with pure HTML (former [SlimUI](https://github.com/hobbyquaker/SlimUI))                                              	                                                                                                    | planned 	    |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **home automation**                                                                                                                                                                                                                                                                       |               |
| [cul](https://github.com/hobbyquaker/ioBroker.cul)                                                        | Different RF Devices ([FS20](http://www.elv.de/fs20-funkschaltsystem.html), [MAX!](http://www.eq-3.de/max-heizungssteuerung.html), FHT, HMS, ...) via [CUL](http://busware.de/tiki-index.php?page=CUL)/[COC](http://busware.de/tiki-index.php?page=COC) and [culfw](http://culfw.de)      | alpha  	    |
| [artnet](https://github.com/iobroker/ioBroker.artnet)                                                     | Control [DMX512](http://en.wikipedia.org/wiki/DMX512) devices via [Art-Net](http://en.wikipedia.org/wiki/Art-Net) (Stage-Lighting: powerful LED RGB Lamps and effects, powerful Dimmers and Switches, moving light effects, fog machines and much more.)                                  | alpha  	    |
| digital                                                                                                   | Control [digitalSTROM](http://www.digitalstrom.com/) devices                                                                                                                                                                                                                              | planned 2016  |
| EnOcean                                                                                                   | Control EnOcean devices via [EnOcean Pi](http://www.element14.com/community/docs/DOC-55169/l/enocean-pi-transforms-raspberry-pi-into-a-wireless-gateway?CMP=PRR-GLO-13-0042)                                                                                                              | planned 2016  |
| [hm-rpc](https://github.com/iobroker/ioBroker.hm-rpc)             	                                    | [Homematic](http://www.homematic.com/) XML-RPC. Control Homematic Devices via Bidcos-Interfaces (unlimited number of CCUs, [RF-LAN-Adapters]() with rfd and [Homegear](http://homegear.eu))                                                             	                                | 0.5.2         |
| [hm-rega](https://github.com/iobroker/ioBroker.hm-rega)           	                                    | Homematic CCU1 or [CCU2](http://www.eq-3.de/produkt-detail-zentralen-und-gateways/items/homematic-zentrale-ccu-2.html) (Logic Layer "ReGaHSS", needed in addition to )                                                                                                                    | 0.2.2         |
| [hue](https://github.com/iobroker/ioBroker.hue)        	                                                | [Philips Hue](http://www.meethue.com) LED bulbs and stripes, Smartlink capable LivingColors and LivingWhites 	                                                                                                                                                                            | 0.4.4         |
| homeeasy                                                                                                  | [Home easy](http://www.elro.eu/en/produkte/cat/home-automation/home-easy-next)	                                                                                                                                                                                                        | planned 2016  |
| [homepilot](https://github.com/GermanBluefox/ioBroker.homepilot)                                          | [Rademacher Homepilot](http://homepilot.rademacher.de/)	                                                                                                                                                                                                                                | planned       |
| [knx](https://github.com/ioBroker/ioBroker.knx)                                                           | [KNX](http://www.knx.org/)                                                                                                                                                                                                                                                                | **beta**      |
| maxcube                                                                                                   | [MAX!](http://www.eq-3.de/max-heizungssteuerung.html) via MAX! Cube                                                                                                                                                                                                                       | planned ?     |
| modbus                                                                                                    | [Modbus](http://www.modbus.org/)                                                                                                                                                                                                                                                          | planned ?     |
| [z-wave](https://github.com/GermanBluefox/ioBroker.z-wave)                                                | [Z-Wave](http://www.z-wave.com/)                                                                                                                                                                                                                                                          | 0.2.2         |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **home entertainment**                                                                                                                                                                                                                                                                    |               |
| dream                                                                                                     | [Dreambox](http://www.dream-multimedia-tv.de/)                                                                                                                                                                                                                                            | planned ?     |
| [irtrans](https://github.com/iobroker/ioBroker.irtrans)                                                   | [IRTrans](http://www.irtrans.de/de/index.php) Send and receive infrared commands                                                                                                                                                                                                          | planned ?     |
| [lirc](https://github.com/iobroker/ioBroker.lirc)                                                         | [LIRC](http://www.lirc.org/) Send and receive infrared commands                                                                                                                                                                                                                           | planned 2016  |
| [mpd](https://github.com/iobroker/ioBroker.mpd)                                                           | [mpd](http://www.musicpd.org/)                                                                                                                                                                                                                                                            | planned       |
| [onkyo](https://github.com/iobroker/ioBroker.onkyo)                                                       | Onkyo AV Receiver                                                                                                                                                                                                                                                                         | 0.1.9         |
| plex                                                                                                      | [Plex](https://plex.tv/)                                                                                                                                                                                                                                                                  | planned ?     |
| [sonos](https://github.com/ioBroker/ioBroker.sonos)                                                       | [Sonos](http://www.sonos.com/)                                                                                                                                                                                                                                                            | 0.1.7         |
| xbmc                                                                                                      | [XBMC](http://xbmc.org/)                                                                                                                                                                                                                                                                  | planned ?     |
| ...          	                                                                                            | ...several TVs and AV Receivers (Pioneer, Onkyo, Yamaha, Samsung, ...)                                                                         	                                                                                                                                        | planned       |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **messaging**                                                                                                                                                                                                                                                                             |               |
| [email](https://github.com/iobroker/ioBroker.email)        	                                            | send emails                                                	                                                                                                                                                                                                                            | 0.1.2         |
| [imap](https://github.com/iobroker/ioBroker.imap)        	                                                | Control ioBroker with emails in clear text over imap protocoll                                                	                                                                                                                                                                        | planned ?	    |
| growl                                                                                                     | [Growl](http://growl.info/)                                                                                                                                                                                                                                                               | planned 2016  |
| nma                                                                                                       | [Notify My Android](http://www.notifymyandroid.com/)                                                                                                                                                                                                                                      | planned ?     |
| [pushover](https://github.com/iobroker/ioBroker.pushover)                                                 | Send [Pushover](https://pushover.net/) notifications                                                    	                                                                                                                                                                                | 0.1.1         |
| [pushbullet](https://github.com/Jens1809/ioBroker.pushbullet)                                             | Send [Pushbullet](https://www.pushbullet.com/) notifications                                                    	                                                                                                                                                                        | 0.0.11        |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **generic protocols**                                                                                                                                                                                                                                                                     |               |
| [mqtt](https://github.com/ioBroker/ioBroker.mqtt)                                                         | [MQTT](http://mqtt.org/)                                                                                                                                                                                                                                                                  | 0.2.x         |
| [ping](https://github.com/ioBroker/ioBroker.ping)                                                         | Use system "ping" command to poll configured IP addresses.                                                                                                                                                                                                                                | 0.1.4         |
| [rest](https://github.com/ioBroker/ioBroker.simple-api)   	                                            | [REST](http://de.wikipedia.org/wiki/Representational_State_Transfer) compliant API                                                            	                                                                                                                                        | 1.0.0         |
| [snmp](https://github.com/iobroker/ioBroker.snmp)                                                         | SNMP                                                                                                                                                                                                                                                                                      | planned ?     |
| [telnet](https://github.com/iobroker/ioBroker.telnet)                                                     | Telnet Client. Connect to a server and send and receive data                                                                                                                                                                                                                              | planned 2016  |
| [xmpp](https://github.com/GermanBluefox/ioBroker.xmpp)                                                    | [XMPP](http://en.wikipedia.org/wiki/XMPP)                                                                                                                                                                                                                                                 | planned 2016  |
| xpl                                                                                                       | [xPL](http://xplproject.org.uk/)                                                                                                                                                                                                                                                          | planned ?     |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **smart metering**                                                                                                                                                                                                                                                                        |               |
| [b-control-em](https://github.com/hobbyquaker/iobroker.b-control-em)                                      | [B-Control Energy Manager](http://www.b-control.com/fileadmin/Webdata/b-control/Uploads/Energiemanagement_PDF/EM300_Datenblatt_rev_100.pdf)  	                                                                                                                                            | **beta**      |
| plugwise                                                                                                  | [Plugwise](http://www.plugwise.com/)                                                                                                                                                                                                                                                      | planned ?     |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **external software**                                                                                                                                                                                                                                                                     |               |
| [graphite](https://github.com/iobroker/ioBroker.nodejs/blob/master/history/README.md)                     | Manage huge amounts of history data and draw charts with [Graphite](http://graphite.wikidot.com/)                                                                                                                                                                                         | **beta**      |
| [nagios](https://github.com/ioBroker/ioBroker.nagios)                                                     | Offer selected objects as [Nagios](http://www.nagios.org/)/[Icinga](https://www.icinga.org/) checks with optional performance data, include nagios hosts as ioBroker objects                                                                                                              | planned 2015  |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **weather**                                                                                                                                                                                                                                                                               |               |
| [dwd](https://github.com/iobroker/ioBroker.dwd)        	                                                | fetch weather warnings from [DWD](http://www.dwd.de) (germany only)                                             	                                                                                                                                                                        | **beta** 	    |
| [yr](https://github.com/iobroker/ioBroker.yr)         	                                                | fetch 48h weather forecasts from [yr.no](http://yr.no) (worldwide)                                     	                                                                                                                                                                                | **beta**      |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **hardware**                                                                                                                                                                                                                                                                              |               |
| arduino                                                                                                   |                                                                                                                                                                                                                                                                                           | planned ?     |
| [gpio](https://github.com/iobroker/ioBroker.gpio)  	                                                    | Use GPIOs, SPI, I2C and 1-Wire on Raspberry Pi, Banana Pi, Cubietruck, ...                                               	                                                                                                                                                                | planned 2016  |
| owfs  	                                                                                                | 1-Wire Devices via (http://owfs.org/)                                 	                                                                                                                                                                                                                | 0.0.2         |
| piface  	                                                                                                | Use [PiFace](http://www.piface.org.uk/)                                     	                                                                                                                                                                                                            | planned ?     |
| tessel  	                                                                                                | Control [Tessel](https://tessel.io) [Modules](https://tessel.io/modules)                    	                                                                                                                                                                                            | planned ?     |
|           	                                                                                            |                                                                                	                                                                                                                                                                                                        |        	    |
|                                                                                                           | **misc**                                                                                                                                                                                                                                                                                  |               |
| [fritz](https://github.com/ruhr70/ioBroker.fritzbox)  	                                                | [FRITZ!Box](http://avm.de/produkte/fritzbox/)                                               	                                                                                                                                                                                            | 0.2.1   	    |
| [geofency](https://github.com/iobroker/ioBroker.geofency)                                                 | Receive [Geofency](http://www.geofency.com/) webhooks                                                        	                                                                                                                                                                            | planned 2016  |
| [ical](https://github.com/ioBroker/ioBroker.ical)                                                         | Fetch iCal/VCALENDAR events                                                                                                                                                                                                                                                               | 1.1.1         |
| noble                                                                                                     | Bluetooth Low Energy devices via [noble](https://github.com/sandeepmistry/noble) ([Parrot Flower Power](http://www.parrot.com/flowerpower), ...)                                                                                                                                          | planned       |
| [text-commands](https://github.com/iobroker/ioBroker.text-commands)  	                                    | Control ioBroker with commands in clear text, like "Switch on the light in the living room". Used by other adapters, like imap or voice control                                               	                                                                                        | planned 	    |
| [text2speech](https://github.com/ioBroker/ioBroker.sayit)                                                 | Text to speech adapter                                                	                                                                                                                                                                                                                | 0.3.13 	    |




#### Security

ioBroker is designed to be accessed by trusted adapters inside trusted networks. This means that usually it is not a
good idea to expose the ioBroker databases, adapters or any smarthome devices directly to the internet or, in general,
to an environment where untrusted clients can directly access these network services. Adapters that offer services
supposed to be exposed to the internet should be handled with care, for example with additional security measures
like VPN, VLAN and reverse proxys.



## Getting Started

#### Operating System and Hardware

[ioBroker.nodejs](https://github.com/iobroker/ioBroker.nodejs/) should run on any hardware and os that runs
[Node.js](http://nodejs.org/) (ARM, x86, Windows, Linux, OSX). Binary builds for CouchDB and Redis are also available
for the ARM and x86 under Windows, Linux and OSX. ioBroker spawns a new Node.js-Process for every adapter instance, so
RAM becomes is a limiting factor. A single adapters memory fingerprint is roundabout 10-60MB. Since CouchDB can create
quite a lot of load a dual core system is beneficial.

We recommend x86 based or ARM based systems like [BananaPi](http://www.bananapi.org/p/product.html) or
[Cubietruck](http://www.exp-tech.de/Mainboards/ARM/Cubietruck.html) using Debian based Linux as operating system.

#### Installation and first steps

* automated installation packages for windows and linux coming soon
* [ioBroker.nodejs manual install](https://github.com/iobroker/ioBroker.nodejs/blob/master/README.md)

#### Community support

* get help in the [ioBroker Forums](http://forum.iobroker.org) (english, german and russian language)


## Docs for (adapter-)developers

* [Core Concepts and Database Schema](doc/SCHEMA.md)
* [Example Javascript/Node.js Adapter](https://github.com/ioBroker/ioBroker.nodejs/blob/master/adapter/example/example.js)
* [ioBroker styleguides](doc/STYLE.md)
* [ioBroker.nodejs Changelog](https://github.com/ioBroker/ioBroker.nodejs/blob/master/CHANGELOG.md)
* [ioBroker.nodejs Roadmap](https://github.com/ioBroker/ioBroker.nodejs/blob/master/ROADMAP.md)
* Direct access to all ioBroker Objects is possible via the CouchDB-Webinterface "Futon": ```http://<couch>:5984/_utils/```
* Use the view selector on the upper right in CouchDB to browse ioBroker objects
* you can use ```redis_cli``` and issue the command ```PSUBSCRIBE *``` to watch all stateChange Events on the Console


## License

The MIT License (MIT)

Copyright (c) 2014-2016 bluefox

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.


